/* Autogenerated by mlir-tblgen; don't manually edit */
#ifdef GEN_PASS_CLASSES

//===----------------------------------------------------------------------===//
// SPIRVCompositeTypeLayout
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class SPIRVCompositeTypeLayoutBase : public ::mlir::OperationPass<ModuleOp> {
public:
  SPIRVCompositeTypeLayoutBase() : ::mlir::OperationPass<ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}
  SPIRVCompositeTypeLayoutBase(const SPIRVCompositeTypeLayoutBase &) : ::mlir::OperationPass<ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  llvm::StringRef getArgument() const override { return "decorate-spirv-composite-type-layout"; }

  /// Returns the derived pass name.
  llvm::StringRef getName() const override { return "SPIRVCompositeTypeLayout"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

protected:
};

//===----------------------------------------------------------------------===//
// SPIRVLowerABIAttributes
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class SPIRVLowerABIAttributesBase : public ::mlir::OperationPass<spirv::ModuleOp> {
public:
  SPIRVLowerABIAttributesBase() : ::mlir::OperationPass<spirv::ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}
  SPIRVLowerABIAttributesBase(const SPIRVLowerABIAttributesBase &) : ::mlir::OperationPass<spirv::ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  llvm::StringRef getArgument() const override { return "spirv-lower-abi-attrs"; }

  /// Returns the derived pass name.
  llvm::StringRef getName() const override { return "SPIRVLowerABIAttributes"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

protected:
};

//===----------------------------------------------------------------------===//
// SPIRVUpdateVCE
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class SPIRVUpdateVCEBase : public ::mlir::OperationPass<spirv::ModuleOp> {
public:
  SPIRVUpdateVCEBase() : ::mlir::OperationPass<spirv::ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}
  SPIRVUpdateVCEBase(const SPIRVUpdateVCEBase &) : ::mlir::OperationPass<spirv::ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  llvm::StringRef getArgument() const override { return "spirv-update-vce"; }

  /// Returns the derived pass name.
  llvm::StringRef getName() const override { return "SPIRVUpdateVCE"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

protected:
};
#undef GEN_PASS_CLASSES
#endif // GEN_PASS_CLASSES
#ifdef GEN_PASS_REGISTRATION
#define GEN_PASS_REGISTRATION_SPIRVCompositeTypeLayout
#define GEN_PASS_REGISTRATION_SPIRVLowerABIAttributes
#define GEN_PASS_REGISTRATION_SPIRVUpdateVCE
#endif // GEN_PASS_REGISTRATION
#ifdef GEN_PASS_REGISTRATION_SPIRVCompositeTypeLayout
::mlir::registerPass("decorate-spirv-composite-type-layout", "Decorate SPIR-V composite type with layout info", []() -> std::unique_ptr<Pass> { return mlir::spirv::createDecorateSPIRVCompositeTypeLayoutPass(); });
#endif // GEN_PASS_REGISTRATION_SPIRVCompositeTypeLayout
#undef GEN_PASS_REGISTRATION_SPIRVCompositeTypeLayout
#ifdef GEN_PASS_REGISTRATION_SPIRVLowerABIAttributes
::mlir::registerPass("spirv-lower-abi-attrs", "Decorate SPIR-V composite type with layout info", []() -> std::unique_ptr<Pass> { return mlir::spirv::createLowerABIAttributesPass(); });
#endif // GEN_PASS_REGISTRATION_SPIRVLowerABIAttributes
#undef GEN_PASS_REGISTRATION_SPIRVLowerABIAttributes
#ifdef GEN_PASS_REGISTRATION_SPIRVUpdateVCE
::mlir::registerPass("spirv-update-vce", "Deduce and attach minimal (version, capabilities, extensions) requirements to spv.module ops", []() -> std::unique_ptr<Pass> { return mlir::spirv::createUpdateVersionCapabilityExtensionPass(); });
#endif // GEN_PASS_REGISTRATION_SPIRVUpdateVCE
#undef GEN_PASS_REGISTRATION_SPIRVUpdateVCE
#ifdef GEN_PASS_REGISTRATION
#undef GEN_PASS_REGISTRATION_SPIRVCompositeTypeLayout
#undef GEN_PASS_REGISTRATION_SPIRVLowerABIAttributes
#undef GEN_PASS_REGISTRATION_SPIRVUpdateVCE
#endif // GEN_PASS_REGISTRATION
#undef GEN_PASS_REGISTRATION
