/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Dialect Declarations                                                       *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


class LLVMAVX512Dialect : public ::mlir::Dialect {
public:
  explicit LLVMAVX512Dialect(::mlir::MLIRContext *context);
  static ::llvm::StringRef getDialectNamespace() { return "llvm_avx512"; }
};
